/*  SpiralSynth
 *  Copyleft (C) 2000 David Griffiths <dave@pawfal.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include <math.h>
#include "NoteTable.h"
#include "SpiralInfo.h"

#ifndef OSC
#define OSC


class Oscillator
{
public:
	Oscillator();
	~Oscillator();

	enum Type{NONE,SQUARE,SAW,NOISE};
	
	void GetOutput(int V,short *data);
	void ModulateFreq(short *data) {m_FreqModBuf=data;}
	void ModulatePulseWidth(short *data) {m_PulseWidthModBuf=data;}
	void ModulateSHLen(short *data) {m_SHModBuf=data;}
	
	void NoteTrigger(int V,int octave,int s,int v);
	void SetOctave(int o) {m_Octave=o;}
	void SetFineFreq(float f) {m_FineFreq=f;}
	void SetPulseWidth(float p) {m_PulseWidth=p;}
	void SetType(Type t) {m_Type=t;}
	void SetSHLen(float s) {m_SHLen=s;}
	void SetModAmount(float s) {m_ModAmount=s;}
	void SetPortmento(float s) {m_PortmentoSpeed=s;}
	void SetVolume(int V, int s) {m_Volume[V]=s;} 
	int   GetOctave() {return m_Octave;}
	float GetFineFreq() {return m_FineFreq;}
	float GetPulseWidth() {return m_PulseWidth;}
	Type  GetType() {return m_Type;}
	float GetSHLen() {return m_SHLen;}
	float GetModAmount() {return m_ModAmount;}
	float GetPortmento() {return m_PortmentoSpeed;}
	int   GetVolume(int V) {return m_Volume[V];} 
	void  Randomise();
	
private:

	// Voice specific parameters
	int   *m_InOctave;
	int   *m_Note;
	int   *m_CyclePos;
	float *m_LastFreq;
	char  *m_Volume;
	
	// Common voice parameters
	Type   m_Type;
	int    m_Octave;
	float  m_FineFreq;
	float  m_PulseWidth;
	float  m_PortmentoSpeed;
	float  m_DesiredFreq;
	float  m_SHLen; 
	float  m_ModAmount;
	short *m_FreqModBuf;
	short *m_PulseWidthModBuf;
	short *m_SHModBuf;
	
	friend istream &operator>>(istream &s, Oscillator &o);
	friend ostream &operator<<(ostream &s, Oscillator &o);

};

istream &operator>>(istream &s, Oscillator &o);
ostream &operator<<(ostream &s, Oscillator &o);

#endif
